@@ -21,14 +21,15 @@
 
 __all__ = [ "Firewall_test" ]
 
-import os.path, sys
+import os.path
+import sys
 import copy
 from firewall import config
 from firewall import functions
 from firewall.core.fw_icmptype import FirewallIcmpType
 from firewall.core.fw_service import FirewallService
-from firewall.core.fw_zone import FirewallZoneIPTables
-from firewall.core.fw_direct import FirewallDirectIPTables
+from firewall.core.fw_zone import FirewallZone
+from firewall.core.fw_direct import FirewallDirect
 from firewall.core.fw_config import FirewallConfig
 from firewall.core.fw_policies import FirewallPolicies
 from firewall.core.fw_ipset import FirewallIPSet
@@ -61,15 +62,10 @@ class Firewall_test(object):
         self.ipset_enabled = False
         self.ipset_supported_types = IPSET_TYPES
 
-        self.available_tables = { }
-        self.available_tables["ipv4"] = [ ]
-        self.available_tables["ipv6"] = [ ]
-        self.available_tables["eb"] = [ ]
-
         self.icmptype = FirewallIcmpType(self)
         self.service = FirewallService(self)
-        self.zone = FirewallZoneIPTables(self)
-        self.direct = FirewallDirectIPTables(self)
+        self.zone = FirewallZone(self)
+        self.direct = FirewallDirect(self)
         self.config = FirewallConfig(self)
         self.policies = FirewallPolicies()
         self.ipset = FirewallIPSet(self)
@@ -252,8 +248,8 @@ class Firewall_test(object):
             try:
                 obj.read()
             except Exception as msg:
-                log.debug1("Failed to load direct rules file '%s': %s",
-                           config.FIREWALLD_DIRECT, msg)
+                log.error("Failed to load direct rules file '%s': %s",
+                          config.FIREWALLD_DIRECT, msg)
         self.config.set_direct(copy.deepcopy(obj))
 
         self._default_zone = self.check_zone(default_zone)
@@ -398,11 +394,6 @@ class Firewall_test(object):
                 self.config.forget_zone(combined_zone.name)
             self.zone.add_zone(combined_zone)
 
-    def get_available_tables(self, ipv):
-        if ipv in [ "ipv4", "ipv6", "eb" ]:
-            return self.available_tables[ipv]
-        return [ ]
-
     def cleanup(self):
         self.icmptype.cleanup()
         self.service.cleanup()
@@ -456,7 +447,7 @@ class Firewall_test(object):
     def check_tcpudp(self, protocol):
         if not protocol:
             raise FirewallError(errors.MISSING_PROTOCOL)
-        if not protocol in [ "tcp", "udp", "sctp", "dccp" ]:
+        if protocol not in [ "tcp", "udp", "sctp", "dccp" ]:
             raise FirewallError(errors.INVALID_PROTOCOL,
                                 "'%s' not in {'tcp'|'udp'|'sctp'|'dccp'}" % \
                                 protocol)
@@ -550,7 +541,6 @@ class Firewall_test(object):
     def set_default_zone(self, zone):
         _zone = self.check_zone(zone)
         if _zone != self._default_zone:
-            _old_dz = self._default_zone
             self._default_zone = _zone
             self._firewalld_conf.set("DefaultZone", _zone)
             self._firewalld_conf.write()
