@@ -31,7 +31,6 @@ from firewall.core.rich import Rich_Rule, Rich_Accept, \
     Rich_IcmpType
 from firewall.core.fw_transaction import FirewallTransaction, \
     FirewallZoneTransaction
-from firewall.core.fw_ifcfg import ifcfg_set_zone_of_interface
 from firewall import errors
 from firewall.errors import FirewallError
 from firewall.fw_types import LastUpdatedOrderedDict
@@ -431,8 +430,6 @@ class FirewallZone(object):
         zone_transaction.add_fail(self.__unregister_interface, _obj,
                                   interface_id)
 
-        zone_transaction.add_post(ifcfg_set_zone_of_interface, zone, interface)
-
         if use_zone_transaction is None:
             zone_transaction.execute(True)
 
@@ -503,12 +500,6 @@ class FirewallZone(object):
         zone_transaction.add_post(self.__unregister_interface, _obj,
                                   interface_id)
 
-        # Do not reset ZONE with ifdown
-        # On reboot or shutdown the zone has been reset to default
-        # if the network service is enabled and controlling the
-        # interface (RHBZ#1381314)
-        #zone_transaction.add_post(ifcfg_set_zone_of_interface, "", interface)
-
         if use_zone_transaction is None:
             zone_transaction.execute(True)
 
